@@ -11,14 +11,12 @@ use Storable 'dclone';
 
 use FusionInventory::Agent::Version;
 use FusionInventory::Agent::Config;
-use FusionInventory::Agent::HTTP::Client::OCS;
 use FusionInventory::Agent::Logger;
 use FusionInventory::Agent::Storage;
 use FusionInventory::Agent::Target::Local;
 use FusionInventory::Agent::Target::Server;
 use FusionInventory::Agent::Tools;
 use FusionInventory::Agent::Tools::Hostname;
-use FusionInventory::Agent::XML::Query::Prolog;
 
 our $VERSION = $FusionInventory::Agent::Version::VERSION;
 my $PROVIDER = $FusionInventory::Agent::Version::PROVIDER;
@@ -31,7 +29,7 @@ sub _versionString {
     my ($VERSION) = @_;
 
     my $string = "$PROVIDER Agent ($VERSION)";
-    if ($VERSION =~ /^\d+\.\d+\.(99\d\d|\d+-dev)$/) {
+    if ($VERSION =~ /^\d+\.\d+\.(99\d\d|\d+-dev|.*-build-?\d+)$/) {
         unshift @{$COMMENTS}, "** THIS IS A DEVELOPMENT RELEASE **";
     }
 
@@ -43,12 +41,10 @@ sub new {
 
     my $self = {
         status  => 'unknown',
-        confdir => $params{confdir},
         datadir => $params{datadir},
         libdir  => $params{libdir},
         vardir  => $params{vardir},
         targets => [],
-        tasks   => []
     };
     bless $self, $class;
 
@@ -60,41 +56,41 @@ sub init {
 
     # Skip create object if still defined (re-init case)
     my $config = $self->{config} || FusionInventory::Agent::Config->new(
-        confdir => $self->{confdir},
         options => $params{options},
+        vardir  => $self->{vardir},
     );
     $self->{config} = $config;
 
-    my $verbosity = $config->{debug} && $config->{debug} == 1 ? LOG_DEBUG  :
-                    $config->{debug} && $config->{debug} == 2 ? LOG_DEBUG2 :
-                                                                LOG_INFO   ;
-
-    my $logger = FusionInventory::Agent::Logger->new(
-        config    => $config,
-        backends  => $config->{logger},
-        verbosity => $verbosity
-    );
+    my $logger = FusionInventory::Agent::Logger->new(config => $config);
     $self->{logger} = $logger;
 
-    $logger->debug("Configuration directory: $self->{confdir}");
+    $logger->debug("Configuration directory: ".$config->confdir());
     $logger->debug("Data directory: $self->{datadir}");
     $logger->debug("Storage directory: $self->{vardir}");
     $logger->debug("Lib directory: $self->{libdir}");
 
     $self->_handlePersistentState();
 
-    $self->_createTargets();
+    # Always reset targets to handle re-init case
+    $self->{targets} = $config->getTargets(
+        logger      => $self->{logger},
+        deviceid    => $self->{deviceid},
+        vardir      => $self->{vardir}
+    );
 
     if (!$self->getTargets()) {
         $logger->error("No target defined, aborting");
         exit 1;
     }
 
+    # Keep program name for Provider inventory as it will be reset in setStatus()
+    FusionInventory::Agent::Task::Inventory::Provider->require();
+    $FusionInventory::Agent::Task::Inventory::Provider::PROGRAM = "$PROGRAM_NAME";
+
     # compute list of allowed tasks
     my %available = $self->getAvailableTasks(disabledTasks => $config->{'no-task'});
     my @tasks = keys %available;
     my @plannedTasks = $self->computeTaskExecutionPlan(\@tasks);
-    $self->{tasksExecutionPlan} = \@plannedTasks;
 
     my %available_lc = map { (lc $_) => $_ } keys %available;
     if (!@tasks) {
@@ -106,13 +102,23 @@ sub init {
     foreach my $task (keys %available) {
         $logger->debug("- $task: $available{$task}");
     }
-    $logger->debug("Planned tasks:");
-    foreach my $task (@{$self->{tasksExecutionPlan}}) {
-        my $task_lc = lc $task;
-        $logger->debug("- $task: " . $available{$available_lc{$task_lc}});
-    }
 
-    $self->{tasks} = \@tasks;
+    foreach my $target ($self->getTargets()) {
+        $logger->debug($target->getType() . " target: " . $target->getName());
+
+        # Register planned tasks by target
+        my @planned = $target->plannedTasks(@plannedTasks);
+
+        if (@planned) {
+            $logger->debug("Planned tasks:");
+            foreach my $task (@planned) {
+                my $task_lc = lc $task;
+                $logger->debug("- $task: " . $available{$available_lc{$task_lc}});
+            }
+        } else {
+            $logger->debug("No planned task");
+        }
+    }
 
     $logger->info("Options 'no-task' and 'tasks' are both used. Be careful that 'no-task' always excludes tasks.")
         if ($self->{config}->isParamArrayAndFilled('no-task') && $self->{config}->isParamArrayAndFilled('tasks'));
@@ -178,7 +184,10 @@ sub runTarget {
     # the prolog dialog must be done once for all tasks,
     # but only for server targets
     my $response;
-    if ($target->isa('FusionInventory::Agent::Target::Server')) {
+    if ($target->isType('server')) {
+
+        return unless FusionInventory::Agent::HTTP::Client::OCS->require();
+
         my $client = FusionInventory::Agent::HTTP::Client::OCS->new(
             logger       => $self->{logger},
             timeout      => $self->{timeout},
@@ -191,6 +200,8 @@ sub runTarget {
             no_compress  => $self->{config}->{'no-compression'},
         );
 
+        return unless FusionInventory::Agent::XML::Query::Prolog->require();
+
         my $prolog = FusionInventory::Agent::XML::Query::Prolog->new(
             deviceid => $self->{deviceid},
         );
@@ -213,7 +224,7 @@ sub runTarget {
         }
     }
 
-    foreach my $name (@{$self->{tasksExecutionPlan}}) {
+    foreach my $name ($target->plannedTasks()) {
         eval {
             $self->runTask($target, $name, $response);
         };
@@ -244,11 +255,14 @@ sub runTaskReal {
 
     my $class = "FusionInventory::Agent::Task::$name";
 
-    $class->require();
+    if (!$class->require()) {
+        $self->{logger}->debug2("$name task module does not compile: $@")
+            if $self->{logger};
+        return;
+    }
 
     my $task = $class->new(
         config       => $self->{config},
-        confdir      => $self->{confdir},
         datadir      => $self->{datadir},
         logger       => $self->{logger},
         target       => $target,
@@ -346,29 +360,6 @@ sub getAvailableTasks {
     return %tasks;
 }
 
-sub _getTaskVersion {
-    my ($self, $module) = @_;
-
-    my $logger = $self->{logger};
-
-    if (!$module->require()) {
-        $logger->debug2("module $module does not compile: $@") if $logger;
-
-        # Don't keep trace of module, only really needed to fix perl 5.8 issue
-        delete $INC{module2file($module)};
-
-        return;
-    }
-
-    my $version;
-    {
-        no strict 'refs';  ## no critic
-        $version = &{$module . '::VERSION'};
-    }
-
-    return $version;
-}
-
 sub _handlePersistentState {
     my ($self) = @_;
 
@@ -476,50 +467,6 @@ sub _makeExecutionPlan {
     return @executionPlan;
 }
 
-sub getTasksExecutionPlan {
-    my ($self) = @_;
-
-    return $self->{tasksExecutionPlan};
-}
-
-sub _createTargets {
-    my ($self) = @_;
-
-    my $config = $self->{config};
-
-    # Always reset targets to handle re-init case
-    $self->{targets} = [];
-
-    # create target list
-    if ($config->{local}) {
-        foreach my $path (@{$config->{local}}) {
-            push @{$self->{targets}},
-                FusionInventory::Agent::Target::Local->new(
-                    logger     => $self->{logger},
-                    deviceid   => $self->{deviceid},
-                    delaytime  => $config->{delaytime},
-                    basevardir => $self->{vardir},
-                    path       => $path,
-                    html       => $config->{html},
-                );
-        }
-    }
-
-    if ($config->{server}) {
-        foreach my $url (@{$config->{server}}) {
-            push @{$self->{targets}},
-                FusionInventory::Agent::Target::Server->new(
-                    logger     => $self->{logger},
-                    deviceid   => $self->{deviceid},
-                    delaytime  => $config->{delaytime},
-                    basevardir => $self->{vardir},
-                    url        => $url,
-                    tag        => $config->{tag},
-                );
-        }
-    }
-}
-
 1;
 __END__
 
@@ -540,10 +487,6 @@ hash:
 
 =over
 
-=item I<confdir>
-
-the configuration directory.
-
 =item I<datadir>
 
 the read-only data directory.
