@@ -7,6 +7,7 @@ use Config;
 use Data::Dumper;
 use Digest::MD5 qw(md5_base64);
 use English qw(-no_match_vars);
+use UNIVERSAL::require;
 use XML::TreePP;
 
 use FusionInventory::Agent::Logger;
@@ -70,6 +71,8 @@ my %fields = (
     PHYSICAL_VOLUMES => [ qw/DEVICE PV_PE_COUNT PV_UUID FORMAT ATTR
                              SIZE FREE PE_SIZE VG_UUID/ ],
     PORTS            => [ qw/CAPTION DESCRIPTION NAME TYPE/ ],
+    POWERSUPPLIES    => [ qw/PARTNUM SERIALNUMBER MANUFACTURER POWER_MAX NAME
+                             HOTREPLACEABLE PLUGGED STATUS LOCATION MODEL/ ],
     PRINTERS         => [ qw/COMMENT DESCRIPTION DRIVER NAME NETWORK PORT
                              RESOLUTION SHARED STATUS ERRSTATUS SERVERNAME
                              SHARENAME PRINTPROCESSOR SERIAL/ ],
@@ -127,11 +130,11 @@ sub new {
     my ($class, %params) = @_;
 
     my $self = {
+        deviceid       => $params{deviceid},
         logger         => $params{logger} || FusionInventory::Agent::Logger->new(),
         fields         => \%fields,
         content        => {
             HARDWARE => {
-                ARCHNAME => $Config{archname},
                 VMSYSTEM => "Physical" # Default value
             },
             VERSIONCLIENT => $FusionInventory::Agent::AGENT_STRING ||
@@ -147,6 +150,56 @@ sub new {
     return $self;
 }
 
+sub getRemote {
+    my ($self) = @_;
+
+    return $self->{_remote} || '';
+}
+
+sub setRemote {
+    my ($self, $task) = @_;
+
+    $self->{_remote} = $task || '';
+
+    return $self->{_remote};
+}
+
+sub getDeviceId {
+    my ($self) = @_;
+
+    return $self->{deviceid} if $self->{deviceid};
+
+    # compute an unique agent identifier based on current time and inventory
+    # hostnale or provider name
+    my $hostname = $self->getHardware('NAME');
+    if ($hostname) {
+        my $workgroup = $self->getHardware('WORKGROUP');
+        $hostname .= "." . $workgroup if $workgroup;
+    } else {
+        FusionInventory::Agent::Tools::Hostname->require();
+
+        eval {
+            $hostname = FusionInventory::Agent::Tools::Hostname::getHostname();
+        };
+    }
+
+    # Fake hostname if no default found
+    $hostname = 'device-by-' . lc($FusionInventory::Agent::Version::PROVIDER) . '-agent'
+        unless $hostname;
+
+    my ($year, $month , $day, $hour, $min, $sec) =
+        (localtime (time))[5, 4, 3, 2, 1, 0];
+
+    return $self->{deviceid} = sprintf "%s-%02d-%02d-%02d-%02d-%02d-%02d",
+        $hostname, $year + 1900, $month + 1, $day, $hour, $min, $sec;
+}
+
+sub getFields {
+    my ($self) = @_;
+
+    return $self->{fields};
+}
+
 sub getContent {
     my ($self) = @_;
 
@@ -592,3 +645,15 @@ compatibility.
 
 At the end of the process IF the inventory was saved
 correctly, the last_state is saved.
+
+=head2 getRemote()
+
+Method to get the parent task remote status.
+
+Returns the string set by setRemote() API or an empty string.
+
+=head2 setRemote([$task])
+
+Method to set or reset the parent task remote status.
+
+Without $task parameter, the API resets the parent remote status to an empty string.
