@@ -2,13 +2,12 @@ package FusionInventory::Agent::Logger;
 
 use strict;
 use warnings;
-use base qw/Exporter/;
 
 use constant {
     LOG_DEBUG2  => 5,
     LOG_DEBUG   => 4,
     LOG_INFO    => 3,
-    LOG_WARNING => 1,
+    LOG_WARNING => 2,
     LOG_ERROR   => 1,
     LOG_NONE    => 0,
 };
@@ -16,19 +15,40 @@ use constant {
 use English qw(-no_match_vars);
 use UNIVERSAL::require;
 
-our @EXPORT = qw/LOG_DEBUG2 LOG_DEBUG LOG_INFO LOG_WARNING LOG_ERROR LOG_NONE/;
+# Package level shared logger config
+my $config;
 
 sub new {
     my ($class, %params) = @_;
 
+    my $first_pass = defined($config) ? 0 : 1 ;
+
+    # Initialize or reset Logger configuration
+    if ($params{config}) {
+        $config = ref($params{config}) eq 'FusionInventory::Agent::Config' ?
+            $params{config}->logger() : $params{config} ;
+    } elsif ($first_pass) {
+        $config = \%params;
+    } else {
+        # Later new creation could be used to update the shared config
+        foreach my $param (keys(%params)) {
+            $config->{$param} = $params{$param};
+        }
+    }
+
+    my $debug = $params{verbosity} || $config->{debug} || 0 ;
+
     my $self = {
-        verbosity => defined $params{verbosity} ? $params{verbosity} : LOG_INFO,
+        verbosity => $debug == 2 ? LOG_DEBUG2 :
+                     $debug == 1 ? LOG_DEBUG  :
+                                   LOG_INFO
     };
     bless $self, $class;
 
     my %backends;
+    my $backends = $params{backends} || $config->{logger} || $params{logger};
     foreach (
-        $params{backends} ? @{$params{backends}} : 'Stderr'
+        $backends ? @{$backends} : 'Stderr'
     ) {
         my $backend = ucfirst($_);
         next if $backends{$backend};
@@ -44,10 +64,11 @@ sub new {
         $self->debug("Logger backend $backend initialised");
         push
             @{$self->{backends}},
-            $package->new(%params);
+            $package->new(%{$config});
     }
 
-    $self->debug($FusionInventory::Agent::VERSION_STRING);
+    # Only log agent version string during the first object creation
+    $self->debug($FusionInventory::Agent::VERSION_STRING) if $first_pass;
 
     return $self;
 }
@@ -55,7 +76,7 @@ sub new {
 sub _log {
     my ($self, %params) = @_;
 
-    # levels: debug2, debug, info, error, fault
+    # levels: debug2, debug, info, warning, error
     my $level = $params{level} || 'info';
     my $message = $params{message};
 
@@ -71,6 +92,12 @@ sub _log {
     }
 }
 
+sub debug_level {
+    my ($self) = @_;
+
+    return $self->{verbosity} > LOG_INFO ? $self->{verbosity} - LOG_INFO : 0;
+}
+
 sub debug2 {
     my ($self, $message) = @_;
 
@@ -130,6 +157,10 @@ FusionInventory::Agent::Logger - FusionInventory logger
 
 This is the logger object.
 
+As this object will mostly be instanciated one time, logger configuration is stored
+at the package level. So if you create a new object but without config, it will
+re-use the saved configuration.
+
 =head1 METHODS
 
 =head2 new(%params)
@@ -141,13 +172,13 @@ hash:
 
 =item I<config>
 
-the agent configuration object, to be passed to backends
+the agent configuration object, to pass debug level and backends parameters
 
-=item I<backends>
+=item I<backends> (deprecated)
 
 a list of backends to use (default: Stderr)
 
-=item I<verbosity>
+=item I<verbosity> (deprecated)
 
 the verbosity level (default: LOG_INFO)
 
@@ -176,3 +207,7 @@ Add a log message with error level.
 =head2 debug_result(%params)
 
 Add a log message with debug level related to an action result.
+
+=head2 debug_level()
+
+Get current logger debug level.
