@@ -2,7 +2,8 @@ package FusionInventory::Agent::Logger::File;
 
 use strict;
 use warnings;
-use base 'FusionInventory::Agent::Logger::Backend';
+
+use parent 'FusionInventory::Agent::Logger::Backend';
 
 use English qw(-no_match_vars);
 use Fcntl qw(:flock);
@@ -12,9 +13,9 @@ sub new {
     my ($class, %params) = @_;
 
     my $self = {
-        logfile         => $params{config}->{logfile},
-        logfile_maxsize => $params{config}->{'logfile-maxsize'} ?
-            $params{config}->{'logfile-maxsize'} * 1024 * 1024 : 0
+        logfile         => $params{'logfile'},
+        logfile_maxsize => $params{'logfile-maxsize'} ?
+            $params{'logfile-maxsize'} * 1024 * 1024 : 0
     };
     bless $self, $class;
 
