@@ -2,25 +2,30 @@ package FusionInventory::Agent::Logger::Syslog;
 
 use strict;
 use warnings;
-use base 'FusionInventory::Agent::Logger::Backend';
+
+use parent 'FusionInventory::Agent::Logger::Backend';
 
 use Sys::Syslog qw(:standard :macros);
 
+use FusionInventory::Agent::Version;
+
 my %syslog_levels = (
     error   => LOG_ERR,
     warning => LOG_WARNING,
     info    => LOG_INFO,
     debug   => LOG_DEBUG,
-    debug2   => LOG_DEBUG
+    debug2  => LOG_DEBUG
 );
 
+my $syslog_name = lc($FusionInventory::Agent::Version::PROVIDER)."-agent";
+
 sub new {
     my ($class, %params) = @_;
 
     my $self = {};
     bless $self, $class;
 
-    openlog("fusinv-agent", 'cons,pid', $params{config}->{logfacility});
+    openlog($syslog_name, 'cons,pid', $params{logfacility} || 'LOG_USER');
 
     return $self;
 }
@@ -28,7 +33,7 @@ sub new {
 sub addMessage {
     my ($self, %params) = @_;
 
-    my $level = $params{level};
+    my $level   = $params{level} || 'info';
     my $message = $params{message};
 
     syslog($syslog_levels{$level}, $message);
