@@ -23,6 +23,10 @@ my %checkType_to_Module = (
     winkeyExists       => "WinKeyExists",
     winkeyMissing      => "WinKeyMissing",
     winkeyEquals       => "WinKeyEquals",
+    winkeyNotEquals    => "WinKeyNotEquals",
+    winvalueExists     => "WinValueExists",
+    winvalueMissing    => "WinValueMissing",
+    winvalueType       => "WinValueType",
 );
 
 sub new {
