@@ -3,6 +3,8 @@ package FusionInventory::Agent::Task::Inventory::MacOS::Drives;
 use strict;
 use warnings;
 
+use parent 'FusionInventory::Agent::Task::Inventory::Module';
+
 use FusionInventory::Agent::Tools;
 use FusionInventory::Agent::Tools::Unix;
 
@@ -43,7 +45,7 @@ sub doInventory {
         my $filesystem = $filesystems{$device};
         next unless $filesystem;
 
-        if ($info->{'Total Size'} =~ /^([.\d]+ \s \S+)/x) {
+        if ($info->{'Total Size'} && $info->{'Total Size'} =~ /^([.\d]+ \s \S+)/x) {
             $filesystem->{TOTAL} = getCanonicalSize($1);
         }
         $filesystem->{SERIAL}     = $info->{'Volume UUID'} ||
