@@ -3,6 +3,8 @@ package FusionInventory::Agent::Task::Inventory::MacOS::Videos;
 use strict;
 use warnings;
 
+use parent 'FusionInventory::Agent::Task::Inventory::Module';
+
 use FusionInventory::Agent::Tools;
 use FusionInventory::Agent::Tools::MacOS;
 
@@ -71,10 +73,15 @@ sub _getDisplays {
                 PCISLOT    => $videoCardInfo->{Slot}
             };
 
-            push @monitors, {
+            my $monitor = {
                 CAPTION     => $displayName,
                 DESCRIPTION => $displayName,
-            }
+            };
+
+            my $serial = getSanitizedString($displayInfo->{'Display Serial Number'});
+            $monitor->{SERIAL} = $serial if $serial;
+
+            push @monitors, $monitor;
         }
     }
 
