@@ -2,12 +2,14 @@ package FusionInventory::Agent::Tools::Hardware;
 
 use strict;
 use warnings;
-use base 'Exporter';
+use parent 'Exporter';
 
 use English qw(-no_match_vars);
 
 use FusionInventory::Agent::Tools;
 use FusionInventory::Agent::Tools::Network;
+use FusionInventory::Agent::Tools::SNMP;
+use FusionInventory::Agent::SNMP::Device;
 
 our @EXPORT = qw(
     getDeviceInfo
@@ -99,49 +101,6 @@ my @sysdescr_rules = (
     },
 );
 
-# rules on model name to reset manufacturer to real vendor
-my %sysmodel_first_word = (
-    'dell'           => { manufacturer => 'Dell', },
-);
-
-# common base variables
-my %base_variables = (
-    CPU          => {
-        oid  => '.1.3.6.1.4.1.9.9.109.1.1.1.1.3.1',
-        type => 'count',
-    },
-    SNMPHOSTNAME => {
-        oid  => [
-            '.1.3.6.1.2.1.1.5.0',
-            '.1.3.6.1.4.1.2699.1.2.1.2.1.1.2.1', # PRINTER-PORT-MONITOR-MIB, ppmPrinterName
-        ],
-        type => 'string',
-    },
-    LOCATION     => {
-        oid  => '.1.3.6.1.2.1.1.6.0',
-        type => 'string',
-    },
-    CONTACT      => {
-        oid  => '.1.3.6.1.2.1.1.4.0',
-        type => 'string',
-    },
-    UPTIME       => {
-        oid  => '.1.3.6.1.2.1.1.3.0',
-        type => 'string',
-    },
-    MEMORY       => {
-        oid  => [
-            '.1.3.6.1.4.1.9.2.1.8.0',
-            '.1.3.6.1.2.1.25.2.3.1.5.1',
-        ],
-        type => 'memory',
-    },
-    RAM          => {
-        oid  => '.1.3.6.1.4.1.9.3.6.6.0',
-        type => 'memory',
-    },
-);
-
 # common interface variables
 my %interface_variables = (
     IFNUMBER         => {
@@ -241,10 +200,16 @@ my %printer_pagecounters_variables = (
     },
     RECTOVERSO => { },
     SCANNED    => {
-        oid   => '.1.3.6.1.4.1.1347.46.10.1.1.5.3' #Kyocera specific counter ( total scan counter)
+        oid   => [
+            '.1.3.6.1.4.1.1347.46.10.1.1.5.3',    #Kyocera specific counter ( total scan counter)
+            '.1.3.6.1.4.1.1602.1.11.1.3.1.4.501'  #Canon specific counter
+        ]
     },
     PRINTTOTAL => {
-        oid   => '.1.3.6.1.4.1.1347.42.3.1.1.1.1.2' #Kyocera specific counter
+        oid   => [
+            '.1.3.6.1.4.1.1347.42.3.1.1.1.1.2',   #Kyocera specific counter
+            '.1.3.6.1.4.1.1602.1.11.1.3.1.4.301'  #Canon specific counter
+        ]
     },
     PRINTBLACK => {
         oid   => '.1.3.6.1.4.1.1347.42.3.1.2.1.1.1.1' #Kyocera specific counter
@@ -253,27 +218,45 @@ my %printer_pagecounters_variables = (
         oid   => '.1.3.6.1.4.1.1347.42.3.1.2.1.1.1.2' #Kyocera specific counter
     },
     COPYTOTAL  => {
-        oid   => '.1.3.6.1.4.1.1347.42.3.1.1.1.1.2' #Kyocera specific counter
+        oid   => [
+            '.1.3.6.1.4.1.1347.42.3.1.1.1.1.2',   #Kyocera specific counter
+            '.1.3.6.1.4.1.1602.1.11.1.3.1.4.101'  #Canon specific counter
+        ]
     },
     COPYBLACK  => {
-        oid   => '.1.3.6.1.4.1.1347.42.3.1.2.1.1.2.1' #Kyocera specific counter
+        oid   => [
+            '.1.3.6.1.4.1.1347.42.3.1.2.1.1.2.1', #Kyocera specific counter
+            '.1.3.6.1.4.1.1602.1.11.1.3.1.4.113'  #Canon specific counter
+        ]
+    },
+    COPYBLACK_A3  => {
+        oid   => '.1.3.6.1.4.1.1602.1.11.1.3.1.4.112' #Canon specific counter
     },
     COPYCOLOR  => {
-        oid   => '.1.3.6.1.4.1.1347.42.3.1.2.1.1.2.2' #Kyocera specific counter
+        oid   => [
+            '.1.3.6.1.4.1.1347.42.3.1.2.1.1.2.2', #Kyocera specific counter
+            '.1.3.6.1.4.1.1602.1.11.1.3.1.4.123'  #Canon specific counter
+        ]
+    },
+    COPYCOLOR_A3  => {
+        oid   => '.1.3.6.1.4.1.1602.1.11.1.3.1.4.122' #Canon specific counter
     },
     FAXTOTAL   => {
         oid   => '.1.3.6.1.4.1.1347.42.3.1.1.1.1.4'  #Kyocera specific counter
     }
 );
 
-sub getDeviceInfo {
+sub _getDevice {
     my (%params) = @_;
 
     my $snmp    = $params{snmp};
     my $datadir = $params{datadir};
     my $logger  = $params{logger};
 
-    my $device;
+    my $device = FusionInventory::Agent::SNMP::Device->new(
+        snmp   => $snmp,
+        logger => $logger
+    );
 
     # manufacturer, type and model identification attempt, using sysObjectID
     my $sysobjectid = $snmp->get('.1.3.6.1.2.1.1.2.0');
@@ -294,7 +277,7 @@ sub getDeviceInfo {
     # if one of them is missing
     my $sysdescr = $snmp->get('.1.3.6.1.2.1.1.1.0');
     if ($sysdescr) {
-        $device->{DESCRIPTION} = _getCanonicalString($sysdescr);
+        $device->{DESCRIPTION} = getCanonicalString($sysdescr);
 
         if (!exists $device->{MANUFACTURER} || !exists $device->{TYPE}) {
             # first word
@@ -320,6 +303,14 @@ sub getDeviceInfo {
         }
     }
 
+    # load supported mibs regarding sysORID list as this list permits to
+    # identify device supported MIBs. But mib supported can also be tested
+    # regarding sysobjectid in some case, so we pass it as argument
+    $device->loadMibSupport($sysobjectid);
+
+    # Set type from MibSupport
+    $device->setType();
+
     # fallback type identification attempt, using type-specific OID presence
     if (!exists $device->{TYPE}) {
          if (
@@ -330,55 +321,12 @@ sub getDeviceInfo {
         }
     }
 
-    # fallback model identification attempt, using type-specific OID value
-    if (!exists $device->{MODEL}) {
-        my $model = exists $device->{TYPE} && $device->{TYPE} eq 'PRINTER' ?
-            $snmp->get('.1.3.6.1.2.1.25.3.2.1.3.1')    :
-            exists $device->{TYPE} && $device->{TYPE} eq 'POWER' ?
-            $snmp->get('.1.3.6.1.2.1.33.1.1.5.0')      : # UPS-MIB
-            $snmp->get('.1.3.6.1.2.1.47.1.1.1.1.13.1') ;
-        $device->{MODEL} = _getCanonicalString($model) if $model;
-    }
+    # Find and set model and manufacturer
+    $device->setModel();
+    $device->setManufacturer();
 
-    # fallback manufacturer identification attempt, using type-agnostic OID
-    if (!exists $device->{MANUFACTURER}) {
-        my $manufacturer = $snmp->get('.1.3.6.1.2.1.43.8.2.1.14.1.1');
-        $device->{MANUFACTURER} = $manufacturer if $manufacturer;
-    }
-
-    # reset manufacturer by rule as real vendor based on first model word
-    if (exists $device->{MODEL}) {
-        my ($first_word) = $device->{MODEL} =~ /(\S+)/;
-        my $result = $sysmodel_first_word{lc($first_word)};
-        if ($result && $result->{manufacturer}) {
-            $device->{MANUFACTURER} = $result->{manufacturer};
-        }
-    }
-
-    # remaining informations
-    foreach my $key (keys %base_variables) {
-        my $variable = $base_variables{$key};
-
-        my $raw_value;
-        if (ref $variable->{oid} eq 'ARRAY') {
-            foreach my $oid (@{$variable->{oid}}) {
-                $raw_value = $snmp->get($oid);
-                last if defined $raw_value;
-            }
-        } else {
-            $raw_value = $snmp->get($variable->{oid});
-        }
-        next unless defined $raw_value;
-
-        my $type = $variable->{type};
-        my $value =
-            $type eq 'memory' ? _getCanonicalMemory($raw_value) :
-            $type eq 'string' ? _getCanonicalString($raw_value) :
-            $type eq 'count'  ? _getCanonicalCount($raw_value)  :
-                                $raw_value;
-
-        $device->{$key} = $value if defined $value;
-    }
+    # Get some common informations like SNMPHOSTNAME, LOCATION, UPTIME and CONTACT
+    $device->setBaseInfos();
 
     # Cleanup some strings from whitespaces
     foreach my $key (qw(MODEL SNMPHOSTNAME LOCATION CONTACT)) {
@@ -388,23 +336,29 @@ sub getDeviceInfo {
         delete $device->{$key} if $device->{$key} eq '';
     }
 
-    my $mac = _getMacAddress($snmp);
-    $device->{MAC} = $mac if $mac;
+    # Find and set Mac address
+    $device->setMacAddress();
 
-    my $serial = _getSerial($snmp, $device->{TYPE});
-    $device->{SERIAL} = $serial if $serial;
+    # Find device serial number
+    $device->setSerial();
 
-    my $firmware = _getFirmware($snmp, $device->{TYPE});
-    $device->{FIRMWARE} = $firmware if $firmware;
+    # Find device firmware
+    $device->setFirmware();
 
-    my $results = $snmp->walk('.1.3.6.1.2.1.4.20.1.1');
-    $device->{IPS}->{IP} =  [
-        sort values %{$results}
-    ] if $results;
+    # Find ip
+    $device->setIp();
 
     return $device;
 }
 
+sub getDeviceInfo {
+    my (%params) = @_;
+
+    my $device = _getDevice(%params);
+
+    return $device->getDiscoveryInfo();
+}
+
 sub _getSysObjectIDInfo {
     my (%params) = @_;
 
@@ -487,133 +441,6 @@ sub _loadSysObjectIDDatabase {
     close $handle;
 }
 
-sub _getSerial {
-    my ($snmp, $type) = @_;
-
-    # Entity-MIB::entPhysicalSerialNum
-    my $entPhysicalSerialNum = $snmp->get_first('.1.3.6.1.2.1.47.1.1.1.1.11');
-    return _getCanonicalSerialNumber($entPhysicalSerialNum)
-        if $entPhysicalSerialNum;
-
-    # Printer-MIB::prtGeneralSerialNumber
-    my $prtGeneralSerialNumber = $snmp->get_first('.1.3.6.1.2.1.43.5.1.1.17');
-    return _getCanonicalSerialNumber($prtGeneralSerialNumber)
-        if $prtGeneralSerialNumber;
-
-    # vendor specific OIDs
-    my @oids = (
-        '.1.3.6.1.4.1.2636.3.1.3.0',             # Juniper-MIB
-        '.1.3.6.1.4.1.248.14.1.1.9.1.10.1',      # Hirschman MIB
-        '.1.3.6.1.4.1.253.8.53.3.2.1.3.1',       # Xerox-MIB
-        '.1.3.6.1.4.1.367.3.2.1.2.1.4.0',        # Ricoh-MIB
-        '.1.3.6.1.4.1.641.2.1.2.1.6.1',          # Lexmark-MIB
-        '.1.3.6.1.4.1.1602.1.2.1.4.0',           # Canon-MIB
-        '.1.3.6.1.4.1.2435.2.3.9.4.2.1.5.5.1.0', # Brother-MIB
-        '.1.3.6.1.4.1.318.1.1.4.1.5.0',          # MasterSwitch-MIB
-        '.1.3.6.1.4.1.6027.3.8.1.1.5.0',         # F10-C-SERIES-CHASSIS-MIB
-        '.1.3.6.1.4.1.6027.3.10.1.2.2.1.12.1',   # FORCE10-SMI
-        '.1.3.6.1.4.1.16378.10000.3.15.0',       # DIGI-Sarian-Monitor
-    );
-    foreach my $oid (@oids) {
-        my $value = $snmp->get($oid);
-        next unless $value;
-        return _getCanonicalSerialNumber($value);
-    }
-
-    return;
-}
-
-sub _getFirmware {
-    my ($snmp, $type) = @_;
-
-    my $entPhysicalSoftwareRev = $snmp->get_first('.1.3.6.1.2.1.47.1.1.1.1.10');
-    return $entPhysicalSoftwareRev if $entPhysicalSoftwareRev;
-
-    my $entPhysicalFirmwareRev = $snmp->get_first('.1.3.6.1.2.1.47.1.1.1.1.9');
-    return $entPhysicalFirmwareRev if $entPhysicalFirmwareRev;
-
-    # vendor specific OIDs
-    my @oids = (
-        '.1.3.6.1.4.1.9.9.25.1.1.1.2.5',         # Cisco / IOS
-        '.1.3.6.1.4.1.248.14.1.1.2.0',           # Hirschman MIB
-        '.1.3.6.1.4.1.2636.3.40.1.4.1.1.1.5.0',  # Juniper-MIB
-    );
-    foreach my $oid (@oids) {
-        my $value = $snmp->get($oid);
-        next unless $value;
-        return _getCanonicalString($value);
-    }
-
-    return;
-}
-
-sub _getMacAddress {
-    my ($snmp) = @_;
-
-    # use BRIDGE-MIB::dot1dBaseBridgeAddress if available
-    my $address_oid = ".1.3.6.1.2.1.17.1.1.0";
-    my $address = _getCanonicalMacAddress($snmp->get($address_oid));
-
-    return $address if $address && $address =~ /^$mac_address_pattern$/;
-
-    # fallback on ports addresses (IF-MIB::ifPhysAddress) if unique
-    my $addresses_oid = ".1.3.6.1.2.1.2.2.1.6";
-    my $addresses = $snmp->walk($addresses_oid);
-
-    # interfaces list with defined ip to use as filter to select shorter mac address list
-    my $ips = $snmp->walk('.1.3.6.1.2.1.4.20.1.2');
-
-    my @all_mac_addresses = ();
-
-    # Try first to obtain shorter mac address list using ip interface list filter
-    @all_mac_addresses = grep { defined } map { $addresses->{$_} } values %{$ips}
-        if (keys(%{$ips}));
-
-    # Finally get all defined mac adresses if ip filtered related list remains empty
-    @all_mac_addresses = grep { defined } values %{$addresses}
-        unless @all_mac_addresses;
-
-    my @valid_mac_addresses =
-        uniq
-        grep { /^$mac_address_pattern$/ }
-        grep { $_ ne '00:00:00:00:00:00' }
-        grep { $_ }
-        map  { _getCanonicalMacAddress($_) }
-        @all_mac_addresses;
-
-    if (@valid_mac_addresses) {
-        return $valid_mac_addresses[0] if @valid_mac_addresses == 1;
-
-        # Compute mac addresses as number and sort them
-        my %macs = map { $_ => _numericMac($_) } @valid_mac_addresses;
-        my @sortedMac = sort { $macs{$a} <=> $macs{$b} } @valid_mac_addresses;
-
-        # Then find first couple of consecutive mac and return first one as this
-        # seems to be the first manufacturer defined mac address
-        while (@sortedMac > 1) {
-            my $currentMac = shift @sortedMac;
-            return $currentMac if ($macs{$currentMac} == $macs{$sortedMac[0]} - 1);
-        }
-    }
-
-    return;
-}
-
-sub _numericMac {
-    my ($mac) = @_;
-
-    my $number = 0;
-    my $multiplicator = 1;
-
-    my @parts = split(':', $mac);
-    while (@parts) {
-        $number += hex(pop(@parts))*$multiplicator;
-        $multiplicator <<= 8 ;
-    }
-
-    return $number;
-}
-
 sub getDeviceFullInfo {
     my (%params) = @_;
 
@@ -621,8 +448,10 @@ sub getDeviceFullInfo {
     my $logger = $params{logger};
 
     # first, let's retrieve basic device informations
-    my $info = getDeviceInfo(%params);
-    return unless $info;
+    my $device = _getDevice(%params);
+    return unless $device;
+
+    my $info = $device->getDiscoveryInfo();
 
     # description is defined as DESCRIPTION for discovery
     # and COMMENTS for inventory
@@ -647,7 +476,10 @@ sub getDeviceFullInfo {
     $info->{TYPE} = $params{type} || $info->{TYPE};
 
     # second, use results to build the object
-    my $device = { INFO => $info };
+    $device->{INFO} = $info ;
+
+    # Set other requested infos
+    $device->setInventoryBaseInfos();
 
     _setGenericProperties(
         device => $device,
@@ -670,9 +502,9 @@ sub getDeviceFullInfo {
     ) if $info->{TYPE} && $info->{TYPE} eq 'NETWORKING';
 
     # external processing for the $device
-    if ($device->{INFO}->{EXTMOD}) {
+    if ($device->{EXTMOD}) {
         runFunction(
-            module   => "FusionInventory::Agent::Tools::Hardware::" . $device->{INFO}->{EXTMOD},
+            module   => __PACKAGE__ . "::" . $device->{EXTMOD},
             function => "run",
             logger   => $logger,
             params   => {
@@ -682,11 +514,11 @@ sub getDeviceFullInfo {
             },
             load     => 1
         );
-
-        # no need to send this to the server
-        delete $device->{INFO}->{EXTMOD};
     }
 
+    # Run any detected mib support
+    $device->runMibSupport();
+
     # convert ports hashref to an arrayref, sorted by interface number
     my $ports = $device->{PORTS}->{PORT};
     if ($ports && %$ports) {
@@ -699,7 +531,7 @@ sub getDeviceFullInfo {
         delete $device->{PORTS};
     }
 
-    return $device;
+    return $device->getInventory();
 }
 
 sub _setGenericProperties {
@@ -733,10 +565,10 @@ sub _setGenericProperties {
         # $prefix.$i = $value, with $i as port id
         while (my ($suffix, $raw_value) = each %{$results}) {
             my $value =
-                $type eq 'mac'      ? _getCanonicalMacAddress($raw_value) :
-                $type eq 'constant' ? _getCanonicalConstant($raw_value)   :
-                $type eq 'string'   ? _getCanonicalString($raw_value)     :
-                $type eq 'count'    ? _getCanonicalCount($raw_value)      :
+                $type eq 'mac'      ? getCanonicalMacAddress($raw_value) :
+                $type eq 'constant' ? getCanonicalConstant($raw_value)   :
+                $type eq 'string'   ? getCanonicalString($raw_value)     :
+                $type eq 'count'    ? getCanonicalCount($raw_value)      :
                                       $raw_value;
             $ports->{$suffix}->{$key} = $value
                 if defined $value && $value ne '';
@@ -762,13 +594,13 @@ sub _setGenericProperties {
         next unless $value;
         # safety checks
         if (! exists $ports->{$value}) {
-            $logger->warning(
+            $logger->debug(
                 "unknown interface $value for IP address $suffix, ignoring"
             ) if $logger;
             next;
         }
         if ($suffix !~ /^$ip_address_pattern$/) {
-            $logger->error("invalid IP address $suffix") if $logger;
+            $logger->debug("invalid IP address $suffix") if $logger;
             next;
         }
         $ports->{$value}->{IP} = $suffix;
@@ -810,7 +642,7 @@ sub _setPrinterProperties {
             $type = $consumable_types{$type_id};
         } else {
             # fallback on description
-            my $description = $descriptions->{$consumable_id};
+            my $description = getCanonicalString($descriptions->{$consumable_id});
             $type =
                 $description =~ /maintenance/i ? 'MAINTENANCEKIT' :
                 $description =~ /fuser/i       ? 'FUSERKIT'       :
@@ -819,21 +651,27 @@ sub _setPrinterProperties {
         }
 
         if (!$type) {
-            $logger->debug("unknown consumable type $type_id") if $logger;
+            $logger->debug("unknown consumable type $type_id: " .
+                (getCanonicalString($descriptions->{$consumable_id}) || "no description")
+            ) if $logger;
             next;
         }
 
         if ($type eq 'TONER' || $type eq 'DRUM' || $type eq 'CARTRIDGE' || $type eq 'DEVELOPER') {
             my $color;
-            if ($color_id) {
-                $color = _getCanonicalString($colors->{$color_id});
+            if ($colors && $color_id) {
+                $color = getCanonicalString($colors->{$color_id});
                 if (!$color) {
-                    $logger->debug("invalid color ID $color_id") if $logger;
+                    $logger->debug("invalid consumable color ID $color_id for : " .
+                        (getCanonicalString($descriptions->{$consumable_id}) || "no description")
+                    ) if $logger;
                     next;
                 }
+                # remove space and following char, XML tag does not accept space
+                $color =~ s/\s.*$//;
             } else {
                 # fallback on description
-                my $description = $descriptions->{$consumable_id};
+                my $description = getCanonicalString($descriptions->{$consumable_id});
                 $color =
                     $description =~ /cyan/i           ? 'cyan'    :
                     $description =~ /magenta/i        ? 'magenta' :
@@ -890,8 +728,8 @@ sub _setPrinterProperties {
             $value = $snmp->get($oid);
         }
         next unless defined $value;
-        if (!_isInteger($value)) {
-            $logger->error("incorrect counter value $value, check $variable->{mapping} mapping") if $logger;
+        if (!isInteger($value)) {
+            $logger->debug("incorrect counter value $value, check $variable->{mapping} mapping") if $logger;
             next;
         }
         $device->{PAGECOUNTERS}->{$key} = $value;
@@ -942,8 +780,8 @@ sub _setNetworkingProperties {
 sub _getPercentValue {
     my ($value1, $value2) = @_;
 
-    return unless defined $value1 && _isInteger($value1);
-    return unless defined $value2 && _isInteger($value2);
+    return unless defined $value1 && isInteger($value1);
+    return unless defined $value2 && isInteger($value2);
     return if $value1 == 0;
 
     return int(
@@ -951,114 +789,6 @@ sub _getPercentValue {
     );
 }
 
-sub _isInteger {
-    $_[0] =~ /^[+-]?\d+$/;
-}
-
-sub _getCanonicalMacAddress {
-    my ($value) = @_;
-
-    return unless $value;
-
-    my $result;
-    my @bytes;
-
-    # packed value, convert from binary to hexadecimal
-    if ($value =~ m/\A [[:ascii:]] \Z/xms) {
-        $value = unpack 'H*', $value;
-    }
-
-    # Check if it's a hex value
-    if ($value =~ /^(?:0x)?([0-9A-F]+)$/i) {
-        @bytes = unpack("(A2)*", $1);
-    } else {
-        @bytes = split(':', $value);
-        # return if bytes are not hex
-        return if grep(!/^[0-9A-F]{1,2}$/i, @bytes);
-    }
-
-    if (scalar(@bytes) == 6) {
-        # it's a MAC
-    } elsif (scalar(@bytes) == 8 &&
-        (($bytes[0] eq '10' && $bytes[1] =~ /^0+/) # WWN 10:00:...
-            || $bytes[0] =~ /^2/)) {               # WWN 2X:XX:...
-    } elsif (scalar(@bytes) < 6) {
-        # make a WWN. prepend "10" and zeroes as necessary
-        while (scalar(@bytes) < 7) { unshift @bytes, '00' }
-        unshift @bytes, '10';
-    } elsif (scalar(@bytes) > 6) {
-        # make a MAC. take 6 bytes from the right
-        @bytes = @bytes[-6 .. -1];
-    }
-
-    $result = join ":", map { sprintf("%02x", hex($_)) } @bytes;
-
-    return if $result eq '00:00:00:00:00:00';
-    return lc($result);
-}
-
-sub _getCanonicalString {
-    my ($value) = @_;
-
-    $value = hex2char($value);
-    return unless defined $value;
-
-    # unquote string
-    $value =~ s/^\\?["']//;
-    $value =~ s/\\?["']$//;
-
-    return unless defined $value;
-
-    # Be sure to work on utf-8 string
-    $value = getUtf8String($value);
-
-    # reduce linefeeds which can be found in descriptions or comments
-    $value =~ s/\p{Control}+\n/\n/g;
-
-    # truncate after first invalid character but keep newline as valid
-    $value =~ s/[^\p{Print}\n].*$//;
-
-    return $value;
-}
-
-sub _getCanonicalSerialNumber {
-    my ($value) = @_;
-
-    $value = hex2char($value);
-    return unless $value;
-
-    $value =~ s/[[:^print:]]//g;
-    $value =~ s/^\s+//;
-    $value =~ s/\s+$//;
-    $value =~ s/\.{2,}//g;
-    return unless $value;
-
-    return $value;
-}
-
-sub _getCanonicalMemory {
-    my ($value) = @_;
-
-    if ($value =~ /^(\d+) KBytes$/) {
-        return int($1 / 1024);
-    } else {
-        return int($value / 1024 / 1024);
-    }
-}
-
-sub _getCanonicalConstant {
-    my ($value) = @_;
-
-    return $value if _isInteger($value);
-    return $1 if $value =~ /\((\d+)\)$/;
-}
-
-sub _getCanonicalCount {
-    my ($value) = @_;
-
-    return _isInteger($value) ? $value  : undef;
-}
-
 sub _getElement {
     my ($oid, $index) = @_;
 
@@ -1173,7 +903,7 @@ sub _addKnownMacAddresses {
     foreach my $port_id (keys %$mac_addresses) {
         # safety check
         if (! exists $ports->{$port_id}) {
-            $logger->error(
+            $logger->debug(
                 "invalid interface ID $port_id while setting known mac " .
                 "addresses, aborting"
             ) if $logger;
@@ -1231,7 +961,7 @@ sub _getKnownMacAddresses {
         next unless defined $interface_id;
 
         my @bytes = split(/\./, $suffix);
-        shift @bytes if @bytes > 6;
+        shift @bytes while @bytes > 6;
 
         push @{$results->{$interface_id}},
             sprintf "%02x:%02x:%02x:%02x:%02x:%02x", @bytes;
@@ -1254,9 +984,9 @@ sub _getKnownMacAddressesDeprecatedOids {
         next unless defined $interface_id;
  
         push @{$results->{$interface_id}},
-            _getCanonicalMacAddress($address2mac->{$suffix});
+            getCanonicalMacAddress($address2mac->{$suffix});
     }
-   
+
     return $results;
 }
 
@@ -1271,8 +1001,8 @@ sub _setConnectedDevices {
         foreach my $interface_id (keys %$lldp_info) {
             # safety check
             if (! exists $ports->{$interface_id}) {
-                $logger->warning(
-                    "unknown interface $interface_id in LLDP info, ignoring"
+                $logger->debug(
+                    "LLDP support: unknown interface $interface_id in LLDP info, ignoring"
                 ) if $logger;
                 next;
             }
@@ -1292,8 +1022,8 @@ sub _setConnectedDevices {
         foreach my $interface_id (keys %$cdp_info) {
             # safety check
             if (! exists $ports->{$interface_id}) {
-                $logger->warning(
-                    "unknown interface $interface_id in CDP info, ignoring"
+                $logger->debug(
+                    "CDP support: unknown interface $interface_id in CDP info, ignoring"
                 ) if $logger;
                 next;
             }
@@ -1310,8 +1040,8 @@ sub _setConnectedDevices {
                     }
                 } else {
                     # undecidable situation
-                    $logger->warning(
-                        "multiple neighbors found by LLDP and CDP for " .
+                    $logger->debug(
+                        "CDP support: multiple neighbors found by LLDP and CDP for " .
                         "interface $interface_id, ignoring"
                     );
                     delete $port->{CONNECTIONS};
@@ -1330,8 +1060,8 @@ sub _setConnectedDevices {
         foreach my $interface_id (keys %$edp_info) {
             # safety check
             if (! exists $ports->{$interface_id}) {
-                $logger->warning(
-                    "unknown interface $interface_id in EDP info, ignoring"
+                $logger->debug(
+                    "EDP support: unknown interface $interface_id in EDP info, ignoring"
                 ) if $logger;
                 next;
             }
@@ -1348,8 +1078,8 @@ sub _setConnectedDevices {
                     }
                 } else {
                     # undecidable situation
-                    $logger->warning(
-                        "multiple neighbors found by LLDP and EDP for " .
+                    $logger->debug(
+                        "EDP support: multiple neighbors found by LLDP and EDP for " .
                         "interface $interface_id, ignoring"
                     );
                     delete $port->{CONNECTIONS};
@@ -1387,18 +1117,42 @@ sub _getLLDPInfo {
     # $prefix.x.y.z = $value
     # whereas y is either a port or an interface id
 
+    # See LldpChassisIdSubtype textual convention in lldp.mib RFC
+    # We only report macAddress='4' at the moment
+    my %not_supported_subtype = (
+        '1' => "chassis component",
+        '2' => "interface alias",
+        '3' => "port component",
+        '5' => "network address",
+        '6' => "interface name",
+        '7' => "local"
+    );
+
     while (my ($suffix, $mac) = each %{$lldpRemChassisId}) {
-        my $sysdescr = _getCanonicalString($lldpRemSysDesc->{$suffix});
-        my $sysname = _getCanonicalString($lldpRemSysName->{$suffix});
+        my $sysdescr = getCanonicalString($lldpRemSysDesc->{$suffix});
+        my $sysname = getCanonicalString($lldpRemSysName->{$suffix});
         next unless ($sysdescr || $sysname);
 
-        # We only support macAddress as LldpChassisIdSubtype at the moment
+        # Skip unexpected suffix format (seen at least on mikrotik devices)
+        if ($suffix =~ /^\d+$/) {
+            $logger->debug2("LLDP support: skipping unsupported suffix interface $suffix")
+                if ($logger);
+            next;
+        }
+
+        # Skip unsupported LldpChassisIdSubtype
         my $subtype = $ChassisIdSubType->{$suffix} || "n/a";
         unless ($subtype eq '4') {
-            $logger->debug(
-                "ChassisId subtype $subtype not supported for <$sysdescr>, value was " .
-                ($mac||"n/a") . ", please report this issue"
-            ) if $logger;
+            if ($logger) {
+                my $info = ($sysname || "no name") . ", " .
+                    (getCanonicalString($mac) || "no chassis id") . ", " .
+                    ($sysdescr || "no description");
+                if ($not_supported_subtype{$subtype}) {
+                    $logger->debug("LLDP support: skipping $not_supported_subtype{$subtype}: $info");
+                } else {
+                    $logger->debug("LLDP support: ChassisId subtype $subtype not supported for <$info>, please report this issue");
+                }
+            }
             next;
         }
 
@@ -1409,13 +1163,13 @@ sub _getLLDPInfo {
         $connection->{SYSNAME} = $sysname if $sysname;
 
         # portId is either a port number or a port mac address,
-        # duplicating chassiId
+        # duplicating chassisId
         my $portId = $lldpRemPortId->{$suffix};
         if ($portId !~ /^0x/ or length($portId) != 14) {
-            $connection->{IFNUMBER} = $portId;
+            $connection->{IFNUMBER} = getCanonicalString($portId);
         }
 
-        my $ifdescr = _getCanonicalString($lldpRemPortDesc->{$suffix});
+        my $ifdescr = getCanonicalString($lldpRemPortDesc->{$suffix});
         $connection->{IFDESCR} = $ifdescr if $ifdescr;
 
         my $id           = _getElement($suffix, -2);
@@ -1450,10 +1204,10 @@ sub _getCDPInfo {
     while (my ($suffix, $ip) = each %{$cdpCacheAddress}) {
         my $interface_id = _getElement($suffix, -2);
         $ip = hex2canonical($ip);
-        next if $ip eq '0.0.0.0';
+        next if (!defined($ip) || $ip eq '0.0.0.0');
 
-        my $sysdescr = _getCanonicalString($cdpCacheVersion->{$suffix});
-        my $model    = _getCanonicalString($cdpCachePlatform->{$suffix});
+        my $sysdescr = getCanonicalString($cdpCacheVersion->{$suffix});
+        my $model    = getCanonicalString($cdpCachePlatform->{$suffix});
         next unless $sysdescr && $model;
 
         my $connection = {
@@ -1467,7 +1221,7 @@ sub _getCDPInfo {
         if ($devicePort =~ /^\d+$/) {
             $connection->{IFNUMBER} = $devicePort;
         } else {
-            $connection->{IFDESCR} = $devicePort;
+            $connection->{IFDESCR} = getCanonicalString($devicePort);
         }
 
         # cdpCacheDeviceId is either remote host name, either remote mac address
@@ -1478,7 +1232,7 @@ sub _getCDPInfo {
                 $connection->{SYSMAC} = lc(alt2canonical($deviceId));
             } else {
                 # otherwise it's an hex-encode hostname
-                $connection->{SYSNAME} = _getCanonicalString($deviceId);
+                $connection->{SYSNAME} = getCanonicalString($deviceId);
             }
         } else {
             $connection->{SYSNAME} = $deviceId;
@@ -1486,14 +1240,14 @@ sub _getCDPInfo {
 
         if ($connection->{SYSNAME} &&
             $connection->{SYSNAME} =~ /^SIP([A-F0-9a-f]*)$/) {
-            $connection->{MAC} = lc(alt2canonical("0x".$1));
+            $connection->{SYSMAC} = lc(alt2canonical("0x".$1));
         }
 
         # warning: multiple neighbors announcement for the same interface
         # usually means a non-CDP aware intermediate equipement
         if ($results->{$interface_id}) {
-            $logger->warning(
-                "multiple neighbors found by CDP for interface $interface_id," .
+            $logger->debug(
+                "CDP support: multiple neighbors found by CDP for interface $interface_id," .
                 " ignoring"
             );
             $blacklist->{$interface_id} = 1;
@@ -1531,7 +1285,7 @@ sub _getEDPInfo {
     # - z1.z2...zz: the vlan name in ASCII
 
     while (my ($suffix, $ip) = each %{$edpNeighborVlanIpAddress}) {
-        next if $ip eq '0.0.0.0';
+        next if (!defined($ip) || $ip eq '0.0.0.0');
 
         my $interface_id = _getElement($suffix, 0);
         my @mac_elements = _getElements($suffix, 3, 8);
@@ -1539,16 +1293,16 @@ sub _getEDPInfo {
 
         my $connection = {
             IP       => $ip,
-            IFDESCR  => $edpNeighborPort->{$short_suffix},
-            SYSNAME  => $edpNeighborName->{$short_suffix},
+            IFDESCR  => getCanonicalString($edpNeighborPort->{$short_suffix}),
+            SYSNAME  => getCanonicalString($edpNeighborName->{$short_suffix}),
             SYSMAC   => sprintf "%02x:%02x:%02x:%02x:%02x:%02x", @mac_elements
         };
 
         # warning: multiple neighbors announcement for the same interface
         # usually means a non-EDP aware intermediate equipement
         if ($results->{$interface_id}) {
-            $logger->warning(
-                "multiple neighbors found by EDP for interface $interface_id," .
+            $logger->debug(
+                "EDP support: multiple neighbors found by EDP for interface $interface_id," .
                 " ignoring"
             );
             $blacklist->{$interface_id} = 1;
@@ -1578,7 +1332,7 @@ sub _setVlans {
     foreach my $port_id (keys %$vlans) {
         # safety check
         if (! exists $ports->{$port_id}) {
-            $logger->error(
+            $logger->debug(
                 "invalid interface ID $port_id while setting vlans, aborting"
             ) if $logger;
             last;
@@ -1663,7 +1417,7 @@ sub _setTrunkPorts {
     foreach my $port_id (keys %$trunk_ports) {
         # safety check
         if (! exists $ports->{$port_id}) {
-            $logger->error(
+            $logger->debug(
                 "invalid interface ID $port_id while setting trunk flag, " .
                 "aborting"
             ) if $logger;
@@ -1741,7 +1495,7 @@ sub _setAggregatePorts {
         foreach my $interface_id (keys %$lacp_info) {
             # safety check
             if (!$ports->{$interface_id}) {
-                $logger->warning(
+                $logger->debug(
                     "unknown interface $interface_id in LACP info, ignoring"
                 ) if $logger;
                 next;
@@ -1755,7 +1509,7 @@ sub _setAggregatePorts {
         foreach my $interface_id (keys %$pagp_info) {
             # safety check
             if (!$ports->{$interface_id}) {
-                $logger->error(
+                $logger->debug(
                     "unknown interface $interface_id in PAGP info, ignoring"
                 ) if $logger;
                 next;
