@@ -3,6 +3,8 @@ package FusionInventory::Agent::Tools::Hardware::Brocade;
 use strict;
 use warnings;
 
+use FusionInventory::Agent::Tools::SNMP qw/getCanonicalMacAddress/;
+
 sub run {
     my (%params) = @_;
 
@@ -61,7 +63,7 @@ sub getConnectedWWNs {
     # .1.3.6.1.2.1.75.1.2.3.1.10.1.2.1 = Hex-STRING: 21 00 00 24 FF 57 5F 18
     #                              ^--- $idx
     while (my ($suffix, $wwn) = each %$fcFxPortNxPortName) {
-        $wwn = FusionInventory::Agent::Tools::Hardware::_getCanonicalMacAddress($wwn);
+        $wwn = getCanonicalMacAddress($wwn);
         next unless $wwn;
 
         my $idx = FusionInventory::Agent::Tools::Hardware::_getElement($suffix, 1);
