@@ -2,7 +2,7 @@ package FusionInventory::Agent::Tools::Win32;
 
 use strict;
 use warnings;
-use base 'Exporter';
+use parent 'Exporter';
 use utf8;
 
 use threads;
@@ -28,11 +28,14 @@ use Win32::TieRegistry (
 
 use FusionInventory::Agent::Tools;
 use FusionInventory::Agent::Tools::Network;
+use FusionInventory::Agent::Tools::Expiration;
+use FusionInventory::Agent::Version;
 
 my $localCodepage;
 
 our @EXPORT = qw(
     is64bit
+    remoteIs64bits
     encodeFromRegistry
     KEY_WOW64_64
     KEY_WOW64_32
@@ -43,15 +46,17 @@ our @EXPORT = qw(
     getLocalCodepage
     runCommand
     FileTimeToSystemTime
-    getUsersFromRegistry
     getAgentMemorySize
     FreeAgentMem
+    getWMIService
+    getFormatedWMIDateTime
 );
 
 my $_is64bits = undef;
 sub is64bit {
     # Cache is64bit() result in a private module variable to avoid a lot of wmi
     # calls and as this value won't change during the service/task lifetime
+    return remoteIs64bits() if _remoteWmi();
     return $_is64bits if $_is64bits;
     return $_is64bits =
         any { $_->{AddressWidth} eq 64 }
@@ -99,26 +104,55 @@ sub _getWMIObjects {
         @_
     );
 
-    my $WMIService = Win32::OLE->GetObject($params{moniker});
+    FusionInventory::Agent::Logger->require();
 
-    # Support alternate moniker if provided and main failed to open
-    unless (defined($WMIService)) {
-        if ($params{altmoniker}) {
+    my $logthat = "";
+    my $logger  = $params{logger} || FusionInventory::Agent::Logger->new();
+
+    my $expiration = getExpirationTime();
+
+    my $WMIService;
+    if (_remoteWmi()) {
+        $WMIService = getWMIService(
+            root => $params{root} || "root\\cimv2",
+            @_
+        );
+        # Support alternate moniker if provided and main failed to open
+        if (!defined($WMIService) && $params{altmoniker}) {
+            $WMIService = getWMIService( moniker => $params{altmoniker} );
+        }
+    } else {
+        $WMIService = Win32::OLE->GetObject($params{moniker});
+        # Support alternate moniker if provided and main failed to open
+        if (!defined($WMIService) && $params{altmoniker}) {
             $WMIService = Win32::OLE->GetObject($params{altmoniker});
         }
-        return unless (defined($WMIService));
     }
 
-    Win32::OLE->use('in');
+    return unless (defined($WMIService));
+
+    my $Instances;
+    if ($params{query}) {
+        $logthat = "$params{query} WMI query";
+        $logger->debug2("Doing $logthat") if $logger;
+        $Instances = $WMIService->ExecQuery($params{query});
+    } else {
+        $logthat = "$params{class} class WMI objects";
+        $logger->debug2("Looking for $logthat") if $logger;
+        $Instances = $WMIService->InstancesOf($params{class});
+    }
+
+    return unless $Instances;
 
     my @objects;
-    foreach my $instance (in(
-        $params{query} ?
-        $WMIService->ExecQuery(@{$params{query}})
-        :
-        $WMIService->InstancesOf($params{class})
-    )) {
+    foreach my $instance ( in $Instances ) {
         my $object;
+
+        if (time >= $expiration) {
+            $logger->info("Timeout reached on $logthat") if $logger;
+            last;
+        }
+
         # Handle Win32::OLE object method, see _getLoggedUsers() method in
         # FusionInventory::Agent::Task::Inventory::Win32::Users as example to
         # use or enhance this feature
@@ -203,6 +237,19 @@ sub getRegistryValue {
         return;
     }
 
+    # Shortcut call in remote wmi case
+    if (_remoteWmi()) {
+        my $win32_ole_dependent_api = {
+            funct => '_getRegistryValueFromWMI',
+            args  => [
+                key     => "$root/$keyName",
+                value   => $valueName
+            ]
+        };
+
+        return _call_win32_ole_dependent_api($win32_ole_dependent_api);
+    }
+
     my $key = _getRegistryKey(
         logger  => $params{logger},
         root    => $root,
@@ -223,13 +270,69 @@ sub getRegistryValue {
     }
 }
 
+sub _getRegistryValueFromWMI {
+    my (%params) = @_;
+
+    my $value = $params{value}
+        or return;
+    my $registry = _getWMIRegistry()
+        or return;
+
+    my ($hKey, $subKey) = $params{key} =~ m{^(HKEY_[^/]+)/(.+)$};
+    return unless $hKey && $subKey;
+
+    # subkey path must be win32 conform
+    $subKey =~ s|/|\\|g;
+
+    Win32::OLE->use('in');
+
+    Win32API::Registry->require();
+
+    Win32::OLE::Variant->require();
+    Win32::OLE::Variant->use(qw/VT_BYREF VT_ARRAY VT_VARIANT/);
+
+    # Using a hashref here is just a convenient way for debugging and keep
+    # computed values between evals
+    my $ret = {
+        path => $subKey
+    };
+
+    eval {
+        # Get expected hKey valeur from registry constants
+        $ret->{hKey} = Win32API::Registry::regConstant($hKey);
+
+        # Uses registry enumeration to list values and their type
+        my $type  = VT_BYREF()|VT_ARRAY()|VT_VARIANT();
+        my $vars  = Win32::OLE::Variant->new($type,[1,1]);
+        my $types = Win32::OLE::Variant->new($type,[1,1]);
+        $ret->{err} = $registry->EnumValues($ret->{hKey}, $subKey, $vars, $types);
+
+        # Find expected value in the list and keep its type but skip when
+        # no values are found to avoid crashing
+        if ($vars->Dim()){
+            my @types = in( $types->Copy->Value() );
+            foreach my $var ( in( $vars->Copy->Value() ) ) {
+                my $type = shift @types;
+                next unless $var && $var eq $value;
+                $ret->{value} = $var;
+                $ret->{type}  = $type;
+                last;
+            }
+        }
+    };
+
+    return unless $ret->{err} == 0 && $ret->{value};
+
+    return _getRegistryKeyValueFromWMI(%{$ret});
+}
+
 sub getRegistryKey {
     my (%params) = @_;
 
+    my $logger = $params{logger};
+
     if (!$params{path}) {
-        $params{logger}->error(
-            "No registry key path provided"
-        ) if $params{logger};
+        $logger->error("No registry key path provided") if $logger;
         return;
     }
 
@@ -238,14 +341,27 @@ sub getRegistryKey {
         $root      = $1;
         $keyName   = $2;
     } else {
-        $params{logger}->error(
-            "Failed to parse '$params{path}'. Does it start with HKEY_?"
-        ) if $params{logger};
+        $logger->error("Failed to parse '$params{path}'. Does it start with HKEY_?")
+            if $logger;
         return;
     }
 
+    # Shortcut call in remote wmi case
+    if (_remoteWmi()) {
+        my $win32_ole_dependent_api = {
+            funct => '_getRegistryKeyFromWMI',
+            args  => [
+                path    => $root,
+                keyName => $keyName,
+                wmiopts => $params{wmiopts}
+            ]
+        };
+
+        return _call_win32_ole_dependent_api($win32_ole_dependent_api);
+    }
+
     return _getRegistryKey(
-        logger  => $params{logger},
+        logger  => $logger,
         root    => $root,
         keyName => $keyName
     );
@@ -270,6 +386,151 @@ sub _getRegistryKey {
     return $key;
 }
 
+sub _getRegistryKeyFromWMI{
+    my (%params) = @_;
+
+    my $keyName = defined $params{keyName} ? $params{keyName} : '';
+
+    my $registry = _getWMIRegistry()
+        or return;
+
+    my ($hKey, $subKey) = $params{path} =~ m{^(HKEY_[^/]+)/?(.*)$};
+    $subKey = "" unless defined $subKey;
+    $subKey .= "/" .$keyName if length $keyName;
+
+    return unless $hKey;
+
+    my %wmiopts = $params{wmiopts} ? %{$params{wmiopts}} : ();
+
+    # subkey path must be win32 conform
+    $subKey =~ s|/|\\|g if $subKey;
+
+    Win32::OLE->use('in');
+
+    Win32API::Registry->require();
+
+    Win32::OLE::Variant->require();
+    Win32::OLE::Variant->use(qw/VT_BYREF VT_ARRAY VT_VARIANT/);
+
+    # Using a hashref here is just a convenient way for debugging and keep
+    # computed values between evals
+    my $ret = {
+        path   => $subKey,
+        result => {}
+    };
+
+    eval {
+        # Get expected hKey value from registry constants
+        $ret->{hKey} = Win32API::Registry::regConstant($hKey);
+    };
+
+    return unless $ret->{hKey};
+
+    # We will try to get all the registry tree by default
+    if (!exists($wmiopts{subkeys}) || $wmiopts{subkeys}) {
+        eval {
+            # Uses registry enumeration to list values and their type
+            my $type  = VT_BYREF()|VT_ARRAY()|VT_VARIANT();
+            my $subs  = Win32::OLE::Variant->new($type,[1,1]);
+            $ret->{err} = $registry->EnumKey($ret->{hKey}, $ret->{path}, $subs);
+
+            # Find expected key in the list if some found
+            $ret->{keys} = [ in( $subs->Copy->Value() ) ]
+                if ($ret->{err} == 0 && $subs->Dim());
+        };
+
+        return unless $ret->{err} == 0;
+    }
+
+    eval {
+        # Uses registry enumeration to list values and their type
+        my $type  = VT_BYREF()|VT_ARRAY()|VT_VARIANT();
+        my $vars  = Win32::OLE::Variant->new($type,[1,1]);
+        my $types = Win32::OLE::Variant->new($type,[1,1]);
+        $ret->{err} = $registry->EnumValues($ret->{hKey}, $ret->{path}, $vars, $types);
+
+        # Find expected value in the list and keep its type but skip when
+        # no values are found to avoid crashing
+        if ($vars->Dim()) {
+            my @types = in( $types->Copy->Value() );
+            foreach my $value ( in( $vars->Copy->Value() ) ) {
+                my $type = shift @types;
+                next unless $value && $type;
+                next if ($wmiopts{values} && ! first { $_ eq $value } @{$wmiopts{values}});
+                $ret->{result}{"/$value"} = _getRegistryKeyValueFromWMI(
+                    hKey    => $ret->{hKey},
+                    path    => $ret->{path},
+                    value   => $value,
+                    type    => $type
+                );
+            }
+        }
+
+        # Populate leafs with recurse calling
+        foreach my $subkey (@{$ret->{keys}}) {
+            $ret->{result}{"$subkey/"} = _getRegistryKeyFromWMI(
+                path    => $params{path}."/".$keyName,
+                keyName => $subkey
+            );
+        }
+    };
+
+    return $ret->{result};
+}
+
+sub _getRegistryKeyValueFromWMI {
+    my (%params) = @_;
+
+    my $registry = _getWMIRegistry()
+        or return;
+
+    Win32API::Registry->require();
+    Win32API::Registry->use(qw/REG_SZ REG_EXPAND_SZ REG_BINARY REG_DWORD REG_MULTI_SZ/);
+
+    Win32::OLE::Variant->require();
+    Win32::OLE::Variant->use(qw/VT_BYREF VT_BSTR VT_I4 VT_ARRAY VT_VARIANT/);
+
+    my $err = 0;
+    my $value;
+    eval {
+        # Retrieve the value for supported types
+        if ($params{type} == REG_SZ()) {                         # REG_SZ
+            my $type = VT_BYREF()|VT_BSTR();
+            my $var  = Win32::OLE::Variant->new($type);
+            $err = $registry->GetStringValue($params{hKey}, $params{path}, $params{value}, $var);
+            $value = $var->Copy->Value();
+
+        } elsif ($params{type} == REG_EXPAND_SZ()) {             # REG_EXPAND_SZ
+            my $type = VT_BYREF()|VT_BSTR();
+            my $var  = Win32::OLE::Variant->new($type);
+            $err = $registry->GetExpandedStringValue($params{hKey}, $params{path}, $params{value}, $var);
+            $value = $var->Copy->Value();
+
+        } elsif ($params{type} == REG_BINARY()) {                # REG_BINARY
+            my $type = VT_BYREF()|VT_ARRAY()|VT_VARIANT();
+            my $var  = Win32::OLE::Variant->new($type,[1,1]);
+            $err = $registry->GetBinaryValue($params{hKey}, $params{path}, $params{value}, $var);
+            $value = join('', map { chr } @{$var->Copy->Value()});
+
+        } elsif ($params{type} == REG_DWORD()) {                 # REG_DWORD
+            my $type = VT_BYREF()|VT_I4();
+            my $var  = Win32::OLE::Variant->new($type,0);
+            $err = $registry->GetDWORDValue($params{hKey}, $params{path}, $params{value}, $var);
+            $value = $var->Copy->Value();
+
+        } elsif ($params{type} == REG_MULTI_SZ()) {              # REG_MULTI_SZ
+            my $type = VT_BYREF()|VT_ARRAY()|VT_VARIANT();
+            my $var  = Win32::OLE::Variant->new($type,[1,1]);
+            $err = $registry->GetMultiStringValue($params{hKey}, $params{path}, $params{value}, $var);
+            $value = $var->Dim() ? $var->Copy->Value() : [];
+        }
+    };
+
+    return unless $err == 0;
+
+    return $value;
+}
+
 sub runCommand {
     my (%params) = (
         timeout => 3600 * 2,
@@ -283,7 +544,9 @@ sub runCommand {
     my $winCwd = Cwd::getcwd();
     $winCwd =~ s{/}{\\}g;
 
-    my ($fh, $filename) = File::Temp::tempfile( "$ENV{TEMP}\\fusinvXXXXXXXXXXX", SUFFIX => '.bat');
+    my $provider = lc($FusionInventory::Agent::Version::PROVIDER);
+    my $template = $ENV{TEMP}."\\".$provider."XXXXXXXXXXX";
+    my ($fh, $filename) = File::Temp::tempfile( $template, SUFFIX => '.bat');
     print $fh "cd \"".$winCwd."\"\r\n";
     print $fh $params{command}."\r\n";
     print $fh "exit %ERRORLEVEL%\r\n";
@@ -320,47 +583,27 @@ sub runCommand {
 sub getInterfaces {
     my (%params) = @_;
 
-    my @properties = qw/Index Description IPEnabled DHCPServer MACAddress
-                           MTU DefaultIPGateway DNSServerSearchOrder IPAddress
-                           IPSubnet/;
-    if ($params{additionalProperties}
-        && $params{additionalProperties}->{NetWorkAdapterConfiguration}) {
-        if (ref($params{additionalProperties}->{NetWorkAdapterConfiguration}) eq 'ARRAY') {
-            push @properties, @{$params{additionalProperties}->{NetWorkAdapterConfiguration}};
-        } else {
-            delete $params{additionalProperties}->{NetWorkAdapterConfiguration};
-        }
-    }
-    
     my @configurations;
-    my @wmiResult =
-            $params{list}
-                && $params{list}->{Win32_NetworkAdapterConfiguration}
-                && ref($params{list}->{Win32_NetworkAdapterConfiguration}) eq 'ARRAY' ?
-        @{$params{list}->{Win32_NetworkAdapterConfiguration}} :
-        getWMIObjects(
+
+    foreach my $object (getWMIObjects(
             class      => 'Win32_NetworkAdapterConfiguration',
-            properties => \@properties
-        );
-    foreach my $object (@wmiResult) {
+            properties => [ qw/
+                Index Description IPEnabled DHCPServer MACAddress MTU
+                DefaultIPGateway DNSServerSearchOrder IPAddress IPSubnet
+                DNSDomain
+                /
+            ]
+    )) {
 
         my $configuration = {
             DESCRIPTION => $object->{Description},
             STATUS      => $object->{IPEnabled} ? "Up" : "Down",
             IPDHCP      => $object->{DHCPServer},
             MACADDR     => $object->{MACAddress},
-            MTU         => $object->{MTU}
+            MTU         => $object->{MTU},
+            DNSDomain   => $object->{DNSDomain}
         };
 
-        if ($params{additionalProperties}
-            && $params{additionalProperties}->{NetWorkAdapterConfiguration}) {
-            for my $prop (@{$params{additionalProperties}->{NetWorkAdapterConfiguration}}) {
-                if (defined $object->{$prop}) {
-                    $configuration->{$prop} = $object->{$prop};
-                }
-            }
-        }
-
         if ($object->{DefaultIPGateway}) {
             $configuration->{IPGATEWAY} = $object->{DefaultIPGateway}->[0];
         }
@@ -381,23 +624,10 @@ sub getInterfaces {
 
     my @interfaces;
 
-    @properties = qw/Index PNPDeviceID Speed PhysicalAdapter AdapterTypeId/;
-    if ($params{additionalProperties}
-        && $params{additionalProperties}->{NetWorkAdapter}) {
-        if (ref($params{additionalProperties}->{NetWorkAdapter}) eq 'ARRAY') {
-            push @properties, @{$params{additionalProperties}->{NetWorkAdapter}};
-        } else {
-            delete $params{additionalProperties}->{NetWorkAdapter};
-        }
-    }
-    @wmiResult =
-            $params{list} && $params{list}->{Win32_NetworkAdapter} ?
-        @{$params{list}->{Win32_NetworkAdapter}} :
-        getWMIObjects(
-            class      => 'Win32_NetworkAdapter',
-            properties => \@properties
-        );
-    foreach my $object (@wmiResult) {
+    foreach my $object (getWMIObjects(
+        class      => 'Win32_NetworkAdapter',
+        properties => [ qw/Index PNPDeviceID Speed PhysicalAdapter GUID/ ]
+    )) {
         # http://comments.gmane.org/gmane.comp.monitoring.fusion-inventory.devel/34
         next unless $object->{PNPDeviceID};
 
@@ -418,26 +648,9 @@ sub getInterfaces {
                     DESCRIPTION => $configuration->{DESCRIPTION},
                     STATUS      => $configuration->{STATUS},
                     MTU         => $configuration->{MTU},
-                    dns         => $configuration->{dns},
+                    dns         => $configuration->{dns}
                 };
 
-                if ($params{additionalProperties}
-                    && $params{additionalProperties}->{NetWorkAdapterConfiguration}) {
-                    for my $prop (@{$params{additionalProperties}->{NetWorkAdapterConfiguration}}) {
-                        if ($configuration->{$prop}) {
-                            $interface->{$prop} = $configuration->{$prop};
-                        }
-                    }
-                }
-                if ($params{additionalProperties}
-                    && $params{additionalProperties}->{NetWorkAdapter}) {
-                    for my $prop (@{$params{additionalProperties}->{NetWorkAdapter}}) {
-                        if ($object->{$prop}) {
-                            $interface->{$prop} = $object->{$prop};
-                        }
-                    }
-                }
-
                 if ($address->[0] =~ /$ip_address_pattern/) {
                     $interface->{IPADDRESS} = $address->[0];
                     $interface->{IPMASK}    = $address->[1];
@@ -456,7 +669,12 @@ sub getInterfaces {
                     );
                 }
 
-                $interface->{SPEED}      = $object->{Speed} / 1_000_000
+                $interface->{GUID} = $object->{GUID}
+                    if $object->{GUID};
+                $interface->{DNSDomain} = $configuration->{DNSDomain}
+                    if $configuration->{DNSDomain};
+
+                $interface->{SPEED}      = int($object->{Speed} / 1_000_000)
                     if $object->{Speed};
                 $interface->{VIRTUALDEV} = _isVirtual($object, $configuration);
 
@@ -475,27 +693,15 @@ sub getInterfaces {
                 dns         => $configuration->{dns}
             };
 
-            $interface->{SPEED}      = $object->{Speed} / 1_000_000
+            $interface->{GUID} = $object->{GUID}
+                if $object->{GUID};
+            $interface->{DNSDomain} = $configuration->{DNSDomain}
+                if $configuration->{DNSDomain};
+
+            $interface->{SPEED}      = int($object->{Speed} / 1_000_000)
                 if $object->{Speed};
             $interface->{VIRTUALDEV} = _isVirtual($object, $configuration);
 
-            if ($params{additionalProperties}
-                && $params{additionalProperties}->{NetWorkAdapterConfiguration}) {
-                for my $prop (@{$params{additionalProperties}->{NetWorkAdapterConfiguration}}) {
-                    if ($configuration->{$prop}) {
-                        $interface->{$prop} = $configuration->{$prop};
-                    }
-                }
-            }
-            if ($params{additionalProperties}
-                && $params{additionalProperties}->{NetWorkAdapter}) {
-                for my $prop (@{$params{additionalProperties}->{NetWorkAdapter}}) {
-                    if ($configuration->{$prop}) {
-                        $interface->{$prop} = $configuration->{$prop};
-                    }
-                }
-            }
-
             push @interfaces, $interface;
         }
 
@@ -664,6 +870,11 @@ sub FreeAgentMem {
 
 my $worker ;
 my $worker_semaphore;
+my $worker_lasterror = [];
+my $wmiService;
+my $wmiLocator;
+my $wmiRegistry;
+my $wmiParams = {};
 
 my @win32_ole_calls : shared;
 
@@ -679,9 +890,61 @@ sub start_Win32_OLE_Worker {
     }
 }
 
+sub setupWorkerLogger {
+    my (%params) = @_;
+
+    # Just create a new Logger object in worker to update default module configuration
+    return defined(FusionInventory::Agent::Logger->new(%params))
+        unless (defined($worker));
+
+    return _call_win32_ole_dependent_api({
+        funct => 'setupWorkerLogger',
+        args  => [ %params ]
+    });
+}
+
+sub getLastError {
+
+    return @{$worker_lasterror}
+        unless (defined($worker));
+
+    return _call_win32_ole_dependent_api({
+        funct => 'getLastError',
+        array => 1,
+        args  => []
+    });
+}
+
+my %known_ole_errors = (
+    scalar(0x80041003)  => "Access denied as the current or specified user name and password were not valid or authorized to make the connection.",
+    scalar(0x8004100E)  => "Invalid namespace",
+    scalar(0x80041064)  => "User credentials cannot be used for local connections",
+    scalar(0x80070005)  => "Access denied",
+    scalar(0x800706BA)  => "The RPC server is unavailable",
+);
+
+sub _keepOleLastError {
+
+    my $lasterror = Win32::OLE->LastError();
+    if ($lasterror) {
+        my $error = 0x80000000 | ($lasterror & 0x7fffffff);
+        # Don't report not accurate and not failure error
+        if ($error != 0x80004005) {
+            $worker_lasterror = [ $error, $known_ole_errors{$error} ];
+            my $logger = FusionInventory::Agent::Logger->new();
+            $logger->debug("Win32::OLE ERROR: ".($known_ole_errors{$error}||$lasterror));
+        }
+    } else {
+        $worker_lasterror = [];
+    }
+}
+
 sub _win32_ole_worker {
     # Load Win32::OLE as late as possible in a dedicated worker
     Win32::OLE->require() or return;
+    # We re-initialize Win32::OLE to later support Events (needed for remote WMI)
+    Win32::OLE->Uninitialize();
+    Win32::OLE->Initialize(Win32::OLE::COINIT_OLEINITIALIZE());
     Win32::OLE::Variant->require() or return;
     Win32::OLE->Option(CP => Win32::OLE::CP_UTF8());
 
@@ -699,6 +962,9 @@ sub _win32_ole_worker {
         if (defined($call)) {
             lock($call);
 
+            # Handle call expiration
+            setExpirationTime(%$call);
+
             # Found requested private function and call it as expected
             my $funct;
             eval {
@@ -712,9 +978,15 @@ sub _win32_ole_worker {
                 $result = &{$funct}(@{$call->{'args'}});
             }
 
+            # Keep Win32::OLE error for later reporting
+            _keepOleLastError() unless $funct == \&getLastError;
+
             # Share back the result
             $call->{'result'} = shared_clone($result);
 
+            # Reset expiration
+            setExpirationTime();
+
             # Signal main thread result is available
             cond_signal($call);
         }
@@ -725,6 +997,18 @@ sub _call_win32_ole_dependent_api {
     my ($call) = @_
         or return;
 
+    # Reset timeout as shared between threads
+    my $now = time;
+    my $expiration = getExpirationTime() || $now + 180;
+
+    # Reduce expiration time by 10% of the remaining time to leave a chance to
+    # the caller to compute any result. By default, the reducing should be 2 seconds.
+    $expiration -= int(($expiration - $now) * 0.01) + 1;
+
+    # Be sure expiration is kept in the future by 10 seconds
+    $expiration = $now + 10 unless $expiration > $now;
+    $call->{expiration} = $expiration;
+
     if (defined($worker)) {
         # Share the expect call
         my $call = shared_clone($call);
@@ -742,10 +1026,11 @@ sub _call_win32_ole_dependent_api {
             # Release semaphore so the worker can continue its job
             $worker_semaphore->up();
 
-            # Now, wait for worker result with one minute timeout
-            my $timeout = time + 60;
+            # Now, wait for worker result, leaving a 1 second grace delay to
+            # give worker a chance to handle the timeout by itself
+            $expiration ++ ;
             while (!exists($call->{'result'})) {
-                last if (!cond_timedwait($call, $timeout, @win32_ole_calls));
+                last if (!cond_timedwait($call, $expiration, @win32_ole_calls));
             }
 
             # Be sure to always block worker on semaphore from now
@@ -753,9 +1038,9 @@ sub _call_win32_ole_dependent_api {
 
             if (exists($call->{'result'})) {
                 $result = $call->{'result'};
-            } else {
+            } elsif (time < $expiration) {
                 # Worker is failing: get back to mono-thread and pray
-                $worker->detach();
+                $worker->detach() if (defined($worker) && !$worker->is_detached());
                 $worker = undef;
                 return _call_win32_ole_dependent_api(@_);
             }
@@ -769,51 +1054,145 @@ sub _call_win32_ole_dependent_api {
         Win32::OLE::Variant->require() or return;
         Win32::OLE->Option(CP => Win32::OLE::CP_UTF8());
 
+        # Handle call expiration
+        setExpirationTime(%$call);
+
         # We come here from worker or if we failed to start worker
         my $funct;
         eval {
             no strict 'refs'; ## no critic (ProhibitNoStrict)
             $funct = \&{$call->{'funct'}};
         };
-        return &{$funct}(@{$call->{'args'}});
+
+        if (exists($call->{'array'}) && $call->{'array'}) {
+            my @results = &{$funct}(@{$call->{'args'}});
+
+            # Keep Win32::OLE error for later reporting
+            _keepOleLastError() unless $funct == \&getLastError;
+
+            # Reset expiration
+            setExpirationTime();
+            return @results;
+        } else {
+            my $result = &{$funct}(@{$call->{'args'}});
+
+            # Keep Win32::OLE error for later reporting
+            _keepOleLastError() unless $funct == \&getLastError;
+
+            # Reset expiration
+            setExpirationTime();
+            return $result;
+        }
     }
 }
 
-sub getUsersFromRegistry {
+sub _remoteWmi {
+    return $wmiParams->{host} ? 1 : 0;
+}
+
+sub remoteIs64bits {
+    return $wmiParams->{is64bits} if $wmiParams->{is64bits};
+    # Retrieve and save is64bit result
+    return $wmiParams->{is64bits} = any { $_->{AddressWidth} eq 64 }
+        getWMIObjects(
+            class       => 'Win32_Processor',
+            properties  => [ qw/AddressWidth/ ]
+        );
+}
+
+sub _connectToService {
     my (%params) = @_;
 
-    my $logger = $params{logger};
-    # ensure native registry access, not the 32 bit view
-    my $flags = is64bit() ? KEY_READ | KEY_WOW64_64 : KEY_READ;
-    my $machKey = $Registry->Open('LMachine', {
-            Access => $flags
-        }) or $logger->error("Can't open HKEY_LOCAL_MACHINE key: $EXTENDED_OS_ERROR");
-    if (!$machKey) {
-        $logger->error("getUsersFromRegistry() : Can't open HKEY_LOCAL_MACHINE key: $EXTENDED_OS_ERROR");
-        return;
-    }
-    $logger->debug2('getUsersFromRegistry() : opened LMachine registry key');
-    my $profileList =
-        $machKey->{"SOFTWARE/Microsoft/Windows NT/CurrentVersion/ProfileList"};
-    next unless $profileList;
-
-    my $userList;
-    foreach my $profileName (keys %$profileList) {
-        $params{logger}->debug2('profileName : ' . $profileName);
-        next unless $profileName =~ m{/$};
-        next unless length($profileName) > 10;
-        my $profilePath = $profileList->{$profileName}{'/ProfileImagePath'};
-        my $sid = $profileList->{$profileName}{'/Sid'};
-        next unless $sid;
-        next unless $profilePath;
-        my $user = basename($profilePath);
-        $userList->{$profileName} = $user;
+    # Be sure to reset known access params in threaded version so
+    # getWMIService won't reset when called from right thread
+    foreach my $param (qw( host user pass root)) {
+        $wmiParams->{$param} = $params{$param};
     }
 
-    if ($params{logger}) {
-        $params{logger}->debug2('getUsersFromRegistry() : retrieved ' . scalar(keys %$userList) . ' users');
+    Win32::OLE->require() or return;
+
+    $wmiLocator = Win32::OLE->CreateObject('WbemScripting.SWbemLocator')
+        or return;
+
+    # Always use en-US (MS_409) locale to avoid localized response
+    $wmiService = $wmiLocator->ConnectServer(
+        $params{host}, $params{root},
+        $params{user}, $params{pass}, 'MS_409'
+    );
+
+    return defined $wmiService;
+}
+
+sub _getWMIRegistry {
+    my (%params) = @_;
+
+    unless ($wmiRegistry) {
+        my $WMIService = getWMIService(root => 'root\\default')
+            or return;
+
+        # If missing on a computer, go in C:\Windows\System32\wbem and run "mofcomp regevent.mof"
+        $wmiRegistry = $WMIService->Get("StdRegProv");
     }
-    return $userList;
+
+    return $wmiRegistry;
+}
+
+sub getWMIService {
+    my (%params) = @_;
+
+    my $host   = $params{host} || $wmiParams->{host} || '127.0.0.1';
+    my $user   = $params{user} || $wmiParams->{user} || '';
+    my $pass   = $params{pass} || $wmiParams->{pass} || '';
+    my $root   = $params{root} || $wmiParams->{root} || 'root\\cimv2';
+
+    # Reset root if found in moniker params
+    if ($params{moniker}) {
+        $params{moniker} =~ s{/}{\\}g;
+        if ($params{moniker} =~ /\\root\\(.*)$/i) {
+            $root = "root\\" . lc($1);
+        }
+    }
+
+    # check if the connection is right otherwise reset it
+    if (!$wmiService || $wmiParams && (
+                $wmiParams->{host} ne $host ||
+                $wmiParams->{user} ne $user ||
+                $wmiParams->{pass} ne $pass ||
+                $wmiParams->{root} ne $root)) {
+
+        $wmiParams = {
+            host    => $host,
+            user    => $user,
+            pass    => $pass,
+            root    => $root
+        };
+
+        my $win32_ole_dependent_api = {
+            funct => '_connectToService',
+            args  => [ %{$wmiParams} ]
+        };
+
+        my @connected = _call_win32_ole_dependent_api($win32_ole_dependent_api);
+
+        # Only set $wmiService as connected status in main thread if worker is active
+        # If no worker is active, $wmiService still decides if connected as it is
+        # set directly in _connectToService()
+        $wmiService = shift @connected
+            if (defined($worker) && @connected);
+    }
+
+    return $wmiService;
+}
+
+sub getFormatedWMIDateTime {
+    my ($datetime) = @_;
+
+    return unless $datetime &&
+        $datetime =~ /^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})\.\d{6}.(\d{3})$/;
+
+    # Timezone in $7 is ignored
+
+    return getFormatedDate($1, $2, $3, $4, $5, $6);
 }
 
 END {
